@@ -1,4 +1,5 @@
 #![forbid(unsafe_code)]
+#![no_std]
 #![doc(html_root_url = "https://docs.rs/range_union_find/0.3.0")]
 
 //! Provides a data structure backed by a vector for unioning ranges of integers.
@@ -16,15 +17,29 @@
 //! ```
 //! 
 //! All the functionality is in the [`IntRangeUnionFind`] struct (though we may add `RangeUnionFind` structs for different element types in the future).
-use std::ops::{Bound, RangeBounds, RangeInclusive};
-use std::ops::{BitOr, Sub, BitAnd, Not, BitXor};
-use std::cmp::{min, max};
+#[cfg(any(feature = "std", test))]
+#[macro_use]
+extern crate std;
+
+extern crate alloc;
+
+use core::ops::{Bound, RangeBounds, RangeInclusive};
+use core::ops::{BitOr, Sub, BitAnd, Not, BitXor};
+use core::cmp::{min, max};
 use num_traits::PrimInt;
+
+use alloc::vec::Vec;
 use sorted_vec::SortedVec;
-use std::iter::FromIterator;
+use core::iter::FromIterator;
 
-use std::fmt;
+use core::fmt;
+use alloc::format;
+use alloc::string::String;
 
+#[cfg(feature = "include_serde")]
+use serde::{Serialize, Deserialize};
+
+#[cfg(feature = "std")]
 use std::error::Error;
 
 /// Enum describing how a range may be invalid.
@@ -46,6 +61,8 @@ impl fmt::Display for RangeOperationError {
         write!(f, "{}", description_str)
     }
 }
+
+#[cfg(feature = "std")]
 impl Error for RangeOperationError {}
 
 /// Enum describing what location an element has in a range.
@@ -109,6 +126,7 @@ fn get_result_wrapped_val<T>(res: Result<T,T>) -> T {
 }
 
 #[derive(Default, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "include_serde", derive(Serialize, Deserialize))]
 /*
  * Stores ranges in the following form:
  * range_storage [a_1, b_1, a_2, b_2, ...]
@@ -365,6 +383,84 @@ where
         }
     }
 
+    /// Returns the range the element is in.
+    /// If the element is in a range, return `Ok(range_with_element)`.
+    /// Otherwise, return `Err(range_between_stored_ranges)`.
+    ///
+    /// *Warning: the `impl RangeBounds<T>` object does not include a `Debug` impl, so `unwrap` and `unwrap_err` cannot be used on the returned result.*
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use range_union_find::*;
+    /// # use std::ops::{Bound, RangeBounds};
+    /// let mut range_obj = IntRangeUnionFind::<i32>::new();
+    /// range_obj.insert_range(&(10..=20));
+    ///
+    /// let one_result = range_obj.find_range_with_element(&1);
+    /// assert!(match one_result {
+    ///     Err(one_err) => {
+    ///         assert_eq!(one_err.start_bound(), Bound::Unbounded);
+    ///         assert_eq!(one_err.end_bound(), Bound::Excluded(&10));
+    ///         true
+    ///     },
+    ///     Ok(_) => false
+    /// });
+    ///
+    /// let twelve_result = range_obj.find_range_with_element(&12);
+    /// assert!(match twelve_result {
+    ///     Ok(twelve_range) => {
+    ///         assert_eq!(twelve_range.start_bound(), Bound::Included(&10));
+    ///         assert_eq!(twelve_range.end_bound(), Bound::Included(&20));
+    ///         true
+    ///     },
+    ///     Err(_) => false
+    /// });
+    ///
+    /// let seventy_result = range_obj.find_range_with_element(&70);
+    /// assert!(match seventy_result {
+    ///     Err(seventy_err) => {
+    ///         assert_eq!(seventy_err.start_bound(), Bound::Excluded(&20));
+    ///         assert_eq!(seventy_err.end_bound(), Bound::Unbounded);
+    ///         true
+    ///     },
+    ///     Ok(_) => false
+    /// });
+    /// ```
+    pub fn find_range_with_element(&self, element: &T) -> Result<impl RangeBounds<T>, impl RangeBounds<T>> {
+        let (element_enum, element_range_id) = self.has_element_enum(element);
+        let range_count = self.range_storage.len() / 2;
+        if element_enum == ContainedType::Exterior {
+            if element_range_id == 0 {
+                return Err((
+                    Bound::Unbounded,
+                    Bound::Excluded(self.range_storage[0])
+                ));
+            } else if element_range_id == range_count {
+                let end_index = 2*element_range_id-1;
+                return Err((
+                    Bound::Excluded(self.range_storage[end_index]),
+                    Bound::Unbounded
+                ));
+            } else {
+                let next_range_start = self.range_storage[2*element_range_id];
+                // 2*(element_range_id-1)+1
+                let prev_range_end = self.range_storage[2*element_range_id-1];
+                return Err((
+                    Bound::Excluded(prev_range_end),
+                    Bound::Excluded(next_range_start)
+                ));
+            }
+        } else {
+            let range_start = self.range_storage[2*element_range_id];
+            let range_end = self.range_storage[2*element_range_id+1];
+            return Ok((
+                Bound::Included(range_start),
+                Bound::Included(range_end)
+            ));
+        }
+    }
+
     /// Inserts the range into the set of ranges.
     ///
     /// # Errors
@@ -652,7 +748,7 @@ impl<T: PrimInt> BitOr<&IntRangeUnionFind<T>> for &IntRangeUnionFind<T> {
     /// Computes the union of the two [`IntRangeUnionFind`] objects.
     fn bitor(self, rhs: &IntRangeUnionFind<T>) -> Self::Output {
         let mut dup_obj = self.clone();
-        dup_obj.extend(rhs.to_collection::<Vec<RangeInclusive<T>>>());
+        dup_obj.extend(rhs.to_collection::<Vec<_>>());
         dup_obj
     }
 }
@@ -662,7 +758,7 @@ impl<T: PrimInt> Sub<&IntRangeUnionFind<T>> for &IntRangeUnionFind<T> {
     /// Subtracts the rhs [`IntRangeUnionFind`] object from the lhs one.
     fn sub(self, rhs: &IntRangeUnionFind<T>) -> Self::Output {
         let mut dup_obj = self.clone();
-        for range in rhs.to_collection::<Vec<RangeInclusive<T>>>() {
+        for range in rhs.to_collection::<Vec<_>>() {
             dup_obj.remove_range(&range).unwrap();
         }
         dup_obj
@@ -697,6 +793,13 @@ impl<T: PrimInt> BitAnd<&IntRangeUnionFind<T>> for &IntRangeUnionFind<T> {
             .into_iter().peekable();
         // We rely on the iteration being in increasing order here
         let mut result_vec: Vec<RangeInclusive<T>> = Vec::new();
+        // min_compare variables only used for asserting the above ordering
+        #[cfg_attr(not(debug_assertions),
+            allow(unused_variables), allow(unused_mut))]
+        let mut min_compare_first = T::min_value();
+        #[cfg_attr(not(debug_assertions),
+            allow(unused_variables), allow(unused_mut))]
+        let mut min_compare_second = T::min_value();
         loop {
             // One iter is out -> no more overlaps possible
             let first_range_option = first_range_iter.peek();
@@ -718,11 +821,24 @@ impl<T: PrimInt> BitAnd<&IntRangeUnionFind<T>> for &IntRangeUnionFind<T> {
                 result_vec.push(overlap_range);
             }
 
-            // Advance the correct iterator
+            // Advance the correct iterator and assert next elements are larger
+            // Wrap asserts in cfg block in case assignment has side effects
             if first_range.1 <= second_range.1 {
                 first_range_iter.next();
+                #[cfg(debug_assertions)]
+                {
+                    debug_assert!(min_compare_first <= first_range.0);
+                    debug_assert!(first_range.0 <= first_range.1);
+                    min_compare_first = first_range.1;
+                }
             } else {
                 second_range_iter.next();
+                #[cfg(debug_assertions)]
+                {
+                    debug_assert!(min_compare_second <= second_range.0);
+                    debug_assert!(second_range.0 <= second_range.1);
+                    min_compare_second = second_range.1;
+                }
             }
         }
         IntRangeUnionFind::from_iter(result_vec.into_iter())
@@ -995,6 +1111,43 @@ mod tests {
         assert!(range_obj.has_range(&(0..8)).unwrap()==OverlapType::Partial(1));
     }
 
+    #[test]
+    fn find_range_with_element_in_triple() {
+        let mut range_obj = IntRangeUnionFind::<i32>::new();
+        range_obj.insert_range(&(0..=4)).unwrap();
+        range_obj.insert_range(&(8..=8)).unwrap();
+        range_obj.insert_range(&(10..=16)).unwrap();
+
+        let minus_one_result = range_obj.find_range_with_element(&-1);
+        if let Err(minus_one_err) = minus_one_result {
+            assert_eq!(get_normalized_range(&minus_one_err).unwrap(),
+                (i32::MIN, -1));
+        } else {
+            panic!("-1 was in range {:?}", range_obj);
+        }
+        let two_result = range_obj.find_range_with_element(&2);
+        if let Ok(two_ok) = two_result {
+            assert_eq!(get_normalized_range(&two_ok).unwrap(),
+                (0, 4));
+        } else {
+            panic!("2 was not in range {:?}", range_obj);
+        }
+        let nine_result = range_obj.find_range_with_element(&9);
+        if let Err(nine_err) = nine_result {
+            assert_eq!(get_normalized_range(&nine_err).unwrap(),
+                (9, 9));
+        } else {
+            panic!("9 was in range {:?}", range_obj);
+        }
+        let twenty_result = range_obj.find_range_with_element(&20);
+        if let Err(twenty_err) = twenty_result {
+            assert_eq!(get_normalized_range(&twenty_err).unwrap(),
+                (17, i32::MAX));
+        } else {
+            panic!("17 was in range {:?}", range_obj);
+        }
+    }
+
     #[test]
     fn insert_contained_range_over_single_range() {
         let mut range_obj_old = IntRangeUnionFind::<u32>::new();
